<?php

namespace includes\api;

use Exception;
use includes\WTV_Plugin;
use WC_Customer;
use WC_Order;
use includes\http\WTV_Request;

class WTV_Customers {

    /**
     * Get customer number by visma_customer_id
     *
     * @param string $visma_customer_id
     * @return mixed|void
     * @throws Exception
     */
	static public function get( $visma_customer_id ) {
		// If no email is sent, fake one that can't possibly be used already
		if ( empty( $visma_customer_id ) || !isset( $visma_customer_id ) )
			return;

		try {
			$response = WTV_Request::get( '/customers/' . $visma_customer_id );
			if( ! empty( $response ) )
				return $response;
		}
		catch( Exception $error ) {
            write_log( $error->getMessage() );
			throw new Exception( $error->getMessage() );
		}
	}

    /**
     * Get customer
     *
     * @param $customer_email
     * @return array
     * @throws Exception
     * @internal param WC_Customer $customer
     */
    static public function get_customer( $customer_email ) {
        $visma_customer_id = get_option('visma_customer_' . $customer_email );

        if( ! $visma_customer_id ){
            throw new Exception( __( 'Customer does not exist', WTV_Plugin::TEXTDOMAIN ) );
        }
        return WTV_Request::get( "/customers/{$visma_customer_id}");
    }

    /**
     * Format Customer.
     *
     * @param WC_Order $order
     * @return mixed
     */
    public static function format_params( $order ){

        $settings = get_option( 'visma_order_sync_settings_' . $order->get_payment_method() );
        $address = $order->get_address();
        $shipping_address2 = $address['country'] == 'US' ? $order->get_shipping_state() : $order->get_shipping_address_2();
        $customer = apply_filters( 'wetail_visma_sync_modify_customer', [
            'ContactPersonEmail'                    => $address['email'],
            'EmailAddress'                          => $address['email'],
            'ContactPersonName'                     => ! empty( $address['company'] ) ? $address['company'] : $address['first_name'] . ' ' . $address['last_name'],
            'DeliveryCustomerName'                  => ! empty( $address['company'] ) ? $address['company'] : $address['first_name'] . ' ' . $address['last_name'],
            'CurrencyCode'                          => $order->get_currency(),
            'Name'                                  => ! empty( $address['company'] ) ? $address['company'] : $address['first_name'] . ' ' . $address['last_name'],
            'IsPrivatePerson'                       => ( ! empty( $address['company'] ) ? false : true ),
            'OrgNr'                                 => get_post_meta( $order->get_id(), '_billing_company_number', true ),
            'VatNumber'                             => get_post_meta( $order->get_id(), '_vat_number', true ),
            'Address1'                              => $address['address_1'],
            'Address2'                              => $address['address_2'],
            'ZipCode'                               => $address['postcode'],
            'City'                                  => $address['city'],
            'CountryCode'                           => $address['country'],
            'ReverseChargeOnConstructionServices'   => false,
            'IsActive'                              => true,
            'InvoiceAddress1'                       => $order->get_billing_address_1(),
            'InvoiceAddress2'                       => $order->get_billing_address_2(),
            'InvoiceCity'                           => $order->get_billing_city(),
            'InvoiceCountryCode'                    => $order->get_billing_country(),
            'InvoicePostalCode'                     => $order->get_billing_postcode(),
            'DeliveryAddress1'                      => $order->get_shipping_address_1(),
            'DeliveryAddress2'                      => $shipping_address2,
            'DeliveryCity'                          => $order->get_shipping_city(),
            'Currency'                              => $order->get_currency(),
            'DeliveryCountryCode'                   => $order->get_shipping_country() ? $order->get_shipping_country() : $order->get_billing_country() ,
            'DeliveryName'                          => $order->get_shipping_first_name().' '.$order->get_shipping_last_name(),
            'DeliveryPostalCode'                    => $order->get_shipping_postcode(),
            'ContactPersonPhone'                    => $address['phone'],
            'Telephone'                             => $address['phone'],
            'ShowPriceVATIncluded'                  => false,
            'TermsOfPaymentId'                      => $settings['terms_of_payment_id'],
        ], $order->get_user());
        return $customer;

    }

    /**
     * Sets Visma customer ID
     *
     * 
     * @param string $email
     * @return int $visma_customer_id
     */
    static private function get_visma_customer_id( $email ) {
        return get_option( 'visma_customer_' . $email );
    }

    /**
     * Sets Visma customer ID
     *
     * 
     * @param WC_Order $order
     * @param $visma_customer_id
     * @return int $visma_customer_id
     */
    static private function set_visma_customer_id( $order, $visma_customer_id ) {
        update_option( 'visma_customer_' . $order->get_billing_email(), $visma_customer_id );
    }

    /**
     * Sync customer
     *
     * 
     * @param WC_Order $order
     * @return int
     * @throws Exception
     */
	static public function sync( $order ) {
	    $customer_params = self::format_params( $order );

	    try{
            $existing_customer = null;
            $visma_customer_id = self::get_visma_customer_id( $order->get_billing_email() );
            if( $visma_customer_id ){
                $existing_customer = self::get( $visma_customer_id );
            }

            // Create new customer
            if( empty( $existing_customer ) ) {
                $response = WTV_Request::post( '/customers',  $customer_params );
                self::set_visma_customer_id( $order, $response->Id );
                $visma_customer_id = $response->Id;
            }
            elseif( ! empty( $existing_customer ) ) {
                $customer_params['CustomerNumber'] = $existing_customer->CustomerNumber;
                try{
                    WTV_Request::put( '/customers/' . $visma_customer_id, $customer_params );

                } catch ( Exception $e ){
                    //TODO GDPR
                }

            }

            return $visma_customer_id;

        }
		catch( Exception $error ) {
			throw new Exception( $error->getMessage() );
		}
	}
}