<?php

namespace includes\api;

use Exception;
use includes\http\WTV_Request;
use includes\utils\WTV_Currency_Converter;
use includes\utils\WTV_Utils;
use WC_Order;
use WC_Order_Refund;
use WC_Product;
use WC_Order_Item;

class WTV_Invoices {


    /**
     * Create an invoice of type credit with order refund total as invoice total
     * @param int $order_id
     * @param int $refund_id
     */
    public static function create_credit_note( $order_id, $refund_id ){
        $order = wc_get_order( $order_id );

        $response = WTV_Request::get('/orders/' . preg_replace('/\D/', '', apply_filters( 'woocommerce_order_number', $order_id, $order ) )  );
        if( !intval( $response->Order->InvoiceReference ) ){
            return;
        }
        if( self::order_is_totally_refunded( $order ) ){
            $response = WTV_Request::put('/customerinvoicedrafts/' . $response->Order->InvoiceReference . '/credit', []);
        }
        else{
            $invoice = array(
                'CustomerNumber' => self::get_customer_number( $order ),
                'InvoiceRows' => self::create_credited_invoice_rows( $order, $refund_id )
            );
            $response = WTV_Request::post( '/customerinvoicedrafts', ['Invoice' => $invoice ]);
        }

        return $response->Invoice->DocumentNumber;
    }

    /**
     * Returns an array of InvoiceRows
     * @param WC_Order $order
     * @param int $refund_id
     * @return array
     */
    public static function create_credited_invoice_rows( $order, $refund_id ){

        $rows = array();
        $refund = new WC_Order_Refund( $refund_id );
        foreach ( $refund->get_items() as $article){
            if( $article instanceof WC_Product ){
                array_push( $rows, array(
                        'ArticleNumber'     => $article->get_sku(),
                        'DeliveredQuantity' => -1,
                        'VAT'               => WTV_Utils::get_wc_tax_rate( $article, $order->get_shipping_country()),
                    )
                );
            }
            elseif ( $article instanceof WC_Order_Item ){
                $product =  $article->get_product();
                array_push( $rows, array(
                        'ArticleNumber'     => $product->get_sku(),
                        'DeliveredQuantity' => $article->get_quantity(),
                        'VAT'               => WTV_Utils::get_wc_tax_rate( $article, $order->get_shipping_country()),
                    )
                );
            }

        }
        return $rows;
    }

    /**
     * Creates an invoice of synced order
     * @param WC_Order $order
     * @param int $order_id
     * @param bool $is_credit_invoice
     * @return null
     * @throws Exception
     * @internal param mixed $vismaOrder
     */
    public static function create_invoice( $order, $order_id, $is_credit_invoice=false ){

        do_action('visma_before_order_sync', array( $order ));

        try {
            $customer_number = WTV_Customers::sync( $order );
            $order_rows = [];
            $order_number = apply_filters( 'woocommerce_order_number', $order_id, $order );
            $address = $order->get_address();

            $visma_invoice = apply_filters( 'wetail_visma_invoice', [
                'CustomerId'                            => $customer_number,
                'Number'                                => preg_replace('/\D/', '', $order_number),
                'IsCreditInvoice'                       => $is_credit_invoice,
                'Status'                                => 1,
                'CustomerIsPrivatePerson'               => true,
                'InvoiceDate'                           => substr( $order->get_date_created(), 0, 10 ), # To cut off order time
                'VATIncluded'                           => apply_filters( 'wetail_visma_sync_order_vat_included', false),
                'CurrencyCode'                          => $order->get_currency(),
                'DeliveryCustomerName'                  => ! empty( $address['company'] ) ? $address['company'] : $address['first_name'] . ' ' . $address['last_name'],
                'DeliveryAddress1'                      => $order->get_shipping_address_1(),
                'DeliveryAddress2'                      => $order->get_shipping_address_2(),
                'DeliveryPostalCode'                    => $order->get_shipping_postcode(),
                'DeliveryCity'                          => $order->get_shipping_city(),
                'DeliveryCountryCode'                   => $order->get_shipping_country() ? $order->get_shipping_country() : $order->get_billing_country(),
                'OurReference'                          => $order->get_shipping_first_name().' '.$order->get_shipping_last_name(),
                'InvoiceAddress1'                       => $order->get_billing_address_1(),
                'InvoiceAddress2'                       => $order->get_billing_address_2(),
                'InvoiceCity'                           => $order->get_billing_city(),
                'InvoiceCountryCode'                    => $order->get_billing_country(),
                'InvoiceCustomerName'                   => ! empty( $address['company'] ) ? $address['company'] : $address['first_name'] . ' ' . $address['last_name'],
                'InvoicePostalCode'                     => $order->get_billing_postcode(),
                'EuThirdParty'                          => false,
                'RotReducedInvoicingType'               => false,
                'ReverseChargeOnConstructionServices'   => false,
                'ReversedConstructionServicesVatFree'   => false,
            ], $order );
            $index = 0;
            foreach( $order->get_items() as $item ) {

                $order_row = self::create_invoice_row( $item, $order, $index );
                $index++;
                if ( isset( $order_row[0]) && is_array( $order_row[0] ) ) {
                    foreach ($order_row as $row) {
                        $order_rows[] = $row;
                    }
                }
                else{
                    $order_rows[] = $order_row;
                }
            }

            $visma_invoice['Rows'] = $order_rows;
            $currency_converter = new WTV_Currency_Converter( $order->get_currency(), $order->get_date_created() );

            //$visma_invoice = $currency_converter->convert();
            $visma_invoice = WTV_Orders::handle_fees( $order, $visma_invoice );

            $visma_order = WTV_Orders::set_way_of_delivery( $order, $visma_invoice );
            # Set order as synced

            $invoice_number = null;


        } catch( Exception $error ) {
            throw new Exception( $error->getMessage() );
        }

        $invoice = apply_filters( 'wetail_visma_invoice_before_visma_submit', $visma_invoice, $order );

        $response = WTV_Request::post( '/customerinvoicedrafts/',  $invoice );

        return $response;
    }

    /**
     * Creates order row array
     *
     * @param WC_Order_Item $item
     * @param WC_Order $order
     * @param $index
     * @return mixed
     */
    public static function create_invoice_row( $item, $order, $index ){

        $product = ( WTV_Orders::item_is_variation( $item )) ? wc_get_product($item->get_variation_id()) : wc_get_product( $item->get_product_id() );
        $product_name = WTV_Orders::get_product_name( $item );
        // Sync product to Visma and return the response
        // But only if the product exists in WC.
        if ( wc_get_product( $product->get_id() ) ) {
            WTV_Products::sync( $product->get_id(), true);
        }

        // Cache quantity in var
        $quantity = $item->get_quantity();
        $subtotal = $order->get_item_subtotal( $item, false, false );

        // Get item total, not rounded
        $total = $order->get_item_total( $item, false, false );

        $order_row = apply_filters( 'wetail_visma_sync_modify_order_row', [
            'LineNumber'                            => $index,
            'ArticleId'                             => WTV_Products::get_visma_article_id( $product ),
            'ArticleNumber'                         => WTV_Products::sanitized_sku( $product->get_sku() ),
            'Text'                                  => WTV_Products::sanitize_description( $product_name ),
            'DeliveredQuantity'                     => $quantity,
            'Quantity'                              => $quantity,
            'IsTextRow'                             => false,
            'IsWorkCost'                            => false,
            'EligibleForReverseChargeOnVat'         => true,
            'ReversedConstructionServicesVatFree'   => false,
            'UnitPrice'                             => $subtotal,
            'DiscountPercentage'                    => WTV_Orders::calculate_item_discount( $subtotal, $total, $quantity ), //TODO
        ], $product, $item );


        // If the above filter returns an array of more rows,
        // we loop through them and add every row to $order_rows[].
        if ( isset( $order_row[0] ) && is_array( $order_row[0] ) ) {
            $order_rows = array();
            foreach ( $order_row as $row ) {
                $order_rows[] = $row;
            }
            return $order_rows;
        }
        else {
            return $order_row;
        }
    }

    /**
     * Returns customer number
     * @param WC_Order $order
     * @return int
     */
    public static function get_customer_number( $order ){
        $customer = WTV_Customers::format_params( $order );
        return WTV_Customers::sync( $customer );
    }

    /**
     * Returns true if order refund total equals order total
     * @param WC_Order $order
     * @return bool
     */
    public static function order_is_totally_refunded( $order ){
        return $order->get_total() == $order->get_total_refunded();
    }

    public static function send_invoice_PDF( $orderId ) {
        $order = wc_get_order( $orderId );

        if ( $order->meta_exists( 'visma_invoice_number' ) ) :
            $response = WTV_Request::get('/customerinvoicedrafts/' . $order->get_meta('visma_invoice_number') . '/email' );
        else :
            //
        endif;
    }
}