<?php

namespace includes\api;

use Exception;
use includes\utils\WTV_Utils;
use includes\WTV_Plugin;
use includes\WTV_Sku_Generator;
use WC_Order;
use WC_Product;
use WC_Product_Variable;
use includes\http\WTV_Request;

class WTV_Products {

    /**
     * Check wether product is synced to Visma
     *
     * TODO: Change to '_visma_synced'
     *
     * @param int $product_id
     * @return bool|mixed
     */
	public static function is_synced( $product_id ) {
		$product = new WC_Product( $product_id );

		if( $variations = self::has_variations($product_id) ) {
			foreach($variations as $variation) {
				if( ! get_post_meta($variation->ID, '_is_synced_to_visma', true) ) return false;
			}
			return true;
		}else{
			return get_post_meta( $product->get_id(), '_is_synced_to_visma', true );
		}
	}

    /**
     * Returns true if product is a variation product
     *
     * @param $product
     * @return mixed
     */
	public static function is_variation( $product ) {
		if( get_class( $product ) == 'WC_Product_Variation' ) {
			return true;
		}
		return false;
	}

    /**
     * Get all product Ids
     *
     * @return mixed
     */
    public static function get_all_product_ids() {
	    global $wpdb;
        $results = $wpdb->get_results( "SELECT ID FROM $wpdb->posts p WHERE p.post_type IN ( 'product', 'product_variation') AND NOT EXISTS ( SELECT * FROM $wpdb->postmeta pm
               WHERE pm.meta_key = '_is_synced_to_visma'
        AND pm.post_id=p.ID)" );

        $arr = [];
        foreach ($results as $result){
            $arr[] = $result->ID;
        }
        return $arr;

    }

    /**
     * Product has variations
     *
     * @param int $product_id
     * @return mixed
     */
	public static function has_variations( $product_id ) {
		return get_posts( [
			'post_parent'   => $product_id,
			'post_status'   => "publish",
			'post_type'     => "product_variation",
			'numberposts'   => -1 # remove the output limitation
		] );
	}

    /**
     * Product exists in Visma
     *
     * @param WC_Product $product
     * @return mixed
     * @throws Exception
     */
	public static function exists_in_visma( $product ) {

        $visma_article_id = $product->get_meta( 'visma_article_id' );
		$response = WTV_Request::get("/articles/" . $visma_article_id );
		if( property_exists( $response, 'Meta' ) ){

            $response = WTV_Request::get('/articles?$filter=Number%20eq%20' . "'" .  $product->get_sku() . "'") ;
            if( property_exists( $response, 'Data' ) &&  intval( $response->Meta->TotalNumberOfResults ) == 1 ){
                self::set_visma_product_id( $product, $response->Data[0]->Id );
                return true;
            }
            else{
                return false;
            }

        }
		return true;

	}

    /**
     * Get account coding
     *
     * @param WC_Product $product
     * @return mixed
     * @throws Exception
     */
    public static function get_account_coding( $product ) {
        return WTV_Visma_Settings::get_account_coding( WTV_Utils::get_wc_tax_rate( $product, 'SE' ) );
    }

    /**
     * Returns product from Visma
     *
     * @param int $product_id
     * @return mixed
     * @throws Exception
     */
    public static function get_product( $product_id ){
        $visma_product_id = wc_get_product( $product_id )->get_meta('visma_article_id' );
        if( ! $visma_product_id ){
            throw new Exception( __( 'Product does not exist', WTV_Plugin::TEXTDOMAIN ) );
        }
        return WTV_Request::get( "/articles/{$visma_product_id}");
    }

    /**
     * Returns product from Visma
     *
     * @param WC_Product $product
     * @return mixed
     * @throws Exception
     */
    public static function get_visma_article_id( $product ){
        $visma_product_id = $product->get_meta( 'visma_article_id' );

        if( ! $visma_product_id ){
            throw new Exception( __( 'Product is missing Visma Id', WTV_Plugin::TEXTDOMAIN ) );
        }
        return $visma_product_id;
    }

    /**
     * Get variation name
     *
     * @param \WC_Order_Item $item
     * @param WC_Product $product
     * @return mixed
     * @throws Exception
     */
    public static function get_variation_name( $item, $product ) {
        $attribute_keys = array();
        $attributes = $product->get_attributes();
        foreach( $attributes as $k =>$v ){
            if( $v['is_variation'] == 1 ){
                array_push( $attribute_keys, $k );
            }
        }
        if( count( $attribute_keys ) > 0){
            $str = '';
            foreach( $attribute_keys as $k ){
                $str .= $item[$k];
            }
            return self::truncate_over_fifty($item['name'] . ' - ' . $str );
        }
        else{
            return self::truncate_over_fifty( $item['name'] );
        }
    }

    /**
     * Sanitize description
     *
     * @param string $description
     * @return string
     */
    public static function sanitize_description( $description ) {
        return substr( sanitize_text_field( preg_replace('/\"|\|/', '', $description ) ), 0, 50  );
    }

    /**
     * Sanitized SKU
     *
     * @param string $sku
     * @return string
     */
    public static function sanitized_sku( $sku ) {
        return preg_replace('/[^A-Za-z0-9-+_\/]/', '', $sku );
    }

    /**
     * Set Visma id
     *
     * @param WC_Product $product
     * @param string $visma_product_id
     * @return string
     */
    public static function set_visma_product_id( $product, $visma_product_id ) {
        $product->add_meta_data( 'visma_article_id', $visma_product_id );
        $product->save();
    }

    /**
     * Send article to Visma
     *
     * @param int $product_id
     * @param array|bool $return_response
     * @param bool $sync_stock
     * @return mixed
     * @throws Exception
     */
	public static function sync( $product_id, $return_response=false, $sync_stock=false ) {

		$product = wc_get_product( $product_id );
		$sku = $product->get_sku();

		if( ! $sku ) {
			if( get_option( 'visma_auto_generate_sku' ) ){
				$sku = WTV_Sku_Generator::set_new_sku( $product );
			}
			else{
				throw new Exception(
					__( 'Product ID {$product_id} is missing SKU.', WTV_Plugin::TEXTDOMAIN , WTV_Plugin::TEXTDOMAIN )
				);
			}
		}

        $sku = WTV_Products::sanitized_sku( $sku );

		if( $variations = self::has_variations( $product_id ) ) {
			if( ! get_option( 'visma_skip_product_variations' ) )
				foreach( $variations as $variation )
					self::sync( $variation->ID );
			if( ! get_option( 'visma_sync_master_product' ) ){
                update_post_meta( $product_id, '_is_synced_to_visma', 1 );
                return $sku;
            }

		}

		$product_title = str_replace('"', "'", $product->get_title() );

		if( get_class( $product ) == 'WC_Product_Variation' ) {
			$variation_title_arr = $product->get_variation_attributes();
			$variation_title = reset( $variation_title_arr );
			$product_title = $product_title . ' - ' . $variation_title;
		}

		$article = apply_filters( 'wetail_visma_sync_product_article', [
			'IsActive'      => true,
			'Number'        => $sku,
			'Name'          => self::sanitize_description( $product_title ),
            'CodingId'      => self::get_account_coding( $product ),
            'UnitId'        => WTV_Visma_Settings::get_standard_unit_id(),
		]);


		if( $sync_stock ){
            $article['StockBalanceAvailable'] = $product->managing_stock() ? $product->get_stock_quantity() : 0;
        }

        if( ! get_option( 'visma_do_not_sync_price' ) ){
            $article['NetPrice'] =  WTV_Utils::format_number( wc_get_price_excluding_tax( $product ) );
        }

		if ( ! self::exists_in_visma( $product )) {
            write_log( "Creating article" );
            $article_response = WTV_Request::post( '/articles',  $article );
            self::set_visma_product_id( $product, $article_response->Id );
		}
		else {
            write_log( "Updating article" );
            $visma_article_id = self::get_visma_article_id( $product );
            $article['ArticleId'] = $visma_article_id;
            $article_response = WTV_Request::put( "/articles/{$visma_article_id}", $article );

		}
        write_log( $article );
		update_post_meta( $product_id, '_is_synced_to_visma', 1 );

		if ( $return_response )
			return $article_response;

		return $sku;
	}

    /**
     * Update stock from Visma
     * @param int $product_id
     * @return mixed
     * @throws Exception
     */
	public static function update_stock_from_visma( $product_id ) {

		$product = wc_get_product($product_id);

		if( $variations = self::has_variations( $product_id ) ) {
			if( ! get_option( 'visma_skip_product_variations' ) ) {
				foreach( $variations as $variation ) {
					self::update_stock_from_visma( $variation->ID );
				}
			}
            		$product->set_manage_stock( false );
            		WC_Product_Variable::sync_stock_status( $product );
			if( ! get_option( 'visma_sync_master_product' ) || get_option( 'visma_sync_master_product' ) != 0 ) return;
		}

		$sku = $product->get_sku();
        $sku = WTV_Products::sanitized_sku( $sku );

		if( ! $sku ){
			throw new Exception(
				__( 'Product ID {$product_id} is missing SKU.', WTV_Plugin::TEXTDOMAIN )
			);
		}

		try {
            $visma_article_id = self::get_visma_article_id( $product );
			$response = WTV_Request::get( '/articles/{$visma_article_id}' );

			if( isset( $response->Article->QuantityInStock ) ) {
                # Set stock status
				$product->set_stock_quantity( $response->Article->QuantityInStock );
            }
		}
		catch( Exception $error ) {
            write_log( $error->getMessage() );
			throw new Exception( 'Product ID {$product_id}: ' . $error->getMessage() );
		}
	}

    /**
     * Update price to Visma
     * @param int $product_id
     * @return mixed
     * @throws Exception
     */
	public static function update_price_from_visma( $product_id ) {

		$product = new WC_Product( $product_id );
		$sku = $product->get_sku();
        $sku = WTV_Products::sanitized_sku( $sku );

		if( ! $sku ) {
			throw new Exception(
				__( 'Product ID {$product_id} is missing SKU.', WTV_Plugin::TEXTDOMAIN )
			);
		}
		$visma_default_price_list = get_option( 'visma_default_price_list' );
		$priceList = empty( $visma_default_price_list ) ? 'A' : get_option( 'visma_default_price_list' );

        https://identity-sandbox.test.vismaonline.com/connect/authorize?client_id=wetail&redirect_uri=https://lagerkolltest.wetail.io/wp-json/visma/callback&scope=ea:api%20offline_access&response_type=code
		$response = WTV_Request::get( '/prices/{$priceList}/{$sku}' );

		if( ! empty( $response->ErrorInformation ) ) {
			throw new Exception( '{$response->ErrorInformation->message} (Felkod: {$response->ErrorInformation->code})' );
		}

		# Update regular price
		update_post_meta( $product_id, '_regular_price', $response->Price->Price );

		if( ! get_post_meta( $product_id, '_sale_price', true ) ) {
			update_post_meta( $product_id, '_price', $response->Price->Price );
		}
	}

	# Get product from Visma
	public static function get( $product ) {
        $visma_article_id = self::get_visma_article_id( $product );

		try {
			$response = WTV_Request::get( "/articles/{$visma_article_id}" );
			return $response;
		}
		catch( Exception $error ) {
            write_log( $error->getMessage() );
			throw new Exception( $error->getMessage() );
		}
	}

    # Truncate string
    public static function truncate_over_fifty( $str ){
        return substr($str , 0, 49);
    }

}