<?php

namespace includes;

use Exception;
use includes\api\WTV_Products;
use includes\api\WTV_Sync_Controller;
use includes\api\WTV_Visma_Settings;
use includes\http\WTV_Auth;
use includes\Wetail\WTV_Credentials;

class WTV_Ajax {
	/**
	 * Send AJAX response
	 *
	 * @param array $data
	 */
	public static function respond( $data = [] )
	{
		$defaults = [
			'error' => false
		];
		$data = array_merge( $defaults, $data );
		die( json_encode( $data ) );
	}

	/**
	 * Send AJAX error
	 *
	 * @param string $message
	 */
	public static function error( $message )
	{
		self::respond( [ 'message' => $message ] );
	}

	/**
	 * Update settings through AJAX
	 */
	public static function update_setting()
	{
		if( ! empty( $_REQUEST['settings'] ) )
			foreach( $_REQUEST['settings'] as $option => $value )
				if( 0 === strpos( $option, 'visma_' ) )
					update_option( $option, $value );

		self::respond();
	}

    /**
     * Process AJAX request
     */
    public static function admin_action()
    {

        $response = [];

        switch ( $_REQUEST['visma_action'] ) {
            case 'visma_update_settings':
                try{
                    WTV_Visma_Settings::get_and_save_settings();
                    $response = [
                        'error' => false,
                        'message' => __( "Initial settings updated.", WTV_Plugin::TEXTDOMAIN )
                    ];
                }
                catch (Exception $error ){
                    $response = [
                        'error' => true,
                        'message' => $error->getMessage()
                    ];
                }

                break;
            case 'visma_get_auth_url':
                $response = [
                    'error' => false,
                    'message' => WTV_Auth::authorize_url()
                ];

        }
        self::respond( $response );
    }

	/**
	 * Process AJAX request
	 */
	public static function process()
	{

		$response = [];

		switch( $_REQUEST['visma_action'] ) {
			// Sync orders to Visma
			case "sync_order":
				if( empty( $_REQUEST['order_id'] ) ) {
					$response = [
						'error' => true,
						'message' => __( "Missing order ID.", WTV_Plugin::TEXTDOMAIN )
					];
					break;
				}

				try {
                    $response = WTV_Sync_Controller::sync( $_REQUEST['order_id'] );
				}
				catch( Exception $error ) {
					$response = [
						'error' => true,
						'message' => $error->getMessage()
					];
				}


				if( empty( $response['error'] ) )
					$response = [
						'error' => false,
						'message' => __( "Order successfully synced.", WTV_Plugin::TEXTDOMAIN )
					];

				break;

			// Sync product to Visma
			case "sync_product":
				if( empty( $_REQUEST['product_id'] ) ) {
					$response = [
						'error' => true,
						'message' => __( "Missing product ID.", WTV_Plugin::TEXTDOMAIN )
					];
					break;
				}

				try {
					WTV_Products::sync( $_REQUEST['product_id'], $return_response = false, $sync_stock=true);
				}
				catch( Exception $error ) {
					$response = [
						'error' => true,
						'message' => $error->getMessage()
					];
				}

				if( empty( $response['error'] ) )
					$response = [
						'error' => false,
						'message' => __( "Product successfully synced.", WTV_Plugin::TEXTDOMAIN )
					];

				break;

			case "send_invoice":
				if( empty( $_REQUEST['order_id'] ) ) {
					$response = [
						'error' => true,
						'message' => __( "Missing order ID.", WTV_Plugin::TEXTDOMAIN )
					];
					break;
				}
				break;

			// Invalid action
			default:
				$response = [
					'error' => true,
					'message' => __( "Invalid action specified.", WTV_Plugin::TEXTDOMAIN )
				];
				break;
		}

		self::respond( $response );
	}

	/**
	 * Do bulk action through AJAX
	 */
	public static function bulk_action()
	{
		global $wpdb;

		$response = [ 'error' => false ];

		if( empty( $_REQUEST['bulk'] ) )
			self::error( "Bulk action is missing." );

		switch( $_REQUEST['bulk'] ) {
			// Upload products to Visma
			case 'visma_sync_products':
				$errors = [];
				$posts = get_posts( [
					'numberposts' => -1,
					'post_type' => 'product',
					'post_status' => 'publish'
				] );
				$synced = 0;

				foreach( $posts as $post ) {
					try {
						WTV_Products::sync( $post->ID, $return_response = false, $sync_stock=false );
					}
					catch( Exception $error ) {
						$errors[] = $error->getMessage();
					}

					$synced++;
				}

				$response['message'] = sprintf( __( "Synced %d articles." . ( ! empty( $errors ) ? "\n\n" . join( "\n", $errors ) : "" ), WTV_Plugin::TEXTDOMAIN ), $synced );
				break;

			// Synchronize inventory from Visma to WooCommerce
			case 'visma_sync_inventory':
				$posts = get_posts( [
					'numberposts' => -1,
					'post_type' => 'product',
					'post_status' => 'publish'
				] );
				$synced = 0;
				$notices = [];

				foreach( $posts as $post ) {
					try {
						WTV_Products::update_stock_from_visma( $post->ID );
						$synced++;
					}
					catch( Exception $error ) {
						$notices[] = $error->getMessage();
					}
				}

				$response['message'] = sprintf( __( "%d WooCommerce products are updated.", WTV_Plugin::TEXTDOMAIN ), $synced );

				if( ! empty( $notices ) )
					$response['message'] .= '\n\n' . join( '\n', $notices );
				break;

            // Sync product to Visma
            case 'visma_sync_all_products':
                try {
                    $response = [
                        'error' => false,
                        'product_ids' => WTV_Products::get_all_product_ids()
                    ];
                    break;
                }
                catch( Exception $error ) {
                    $response = [
                        'error' => true,
                        'message' => $error->getMessage()
                    ];
                }

                if( empty( $response['error'] ) )
                    $response = [
                        'error' => false,
                        'message' => __( "Product successfully synced.", WTV_Plugin::TEXTDOMAIN )
                    ];

                break;
		}
		
		self::respond( $response );
	}

	# Check API key
	public static function check_api_key()
	{
		if( ! get_option( 'visma_api_key' ) )
			self::error( __( "API-key is empty. Forgot to save changes?", WTV_Plugin::TEXTDOMAIN ) );

		if ( WTV_Credentials::Check() ) {
			self::respond( [ 'message' => __( "API-key is valid.", WTV_Plugin::TEXTDOMAIN ) ] );
		}else{
			self::error( __( "API-key is invalid.", WTV_Plugin::TEXTDOMAIN ) );
		}
	}
}