<?php

namespace includes\http;

require(WTV_PATH . '/vendor/autoload.php');

use GuzzleHttp\Client;
use includes\WTV_Plugin;
use Exception;

class WTV_Auth
{

    const BASE_TEST_URI = 'https://identity-sandbox.test.vismaonline.com/connect/';
    const BASE_URI = 'https://identity.vismaonline.com/connect/';

    /**
     * Performs authenticate request
     * @param string $code
     * @return bool
     * @throws Exception
     */
    public static function authenticate( $code ){
        $uri = get_option('visma_test') ? self::BASE_TEST_URI : self::BASE_URI;
        $client = new Client( ['base_uri' => $uri ] );
        $options = array();
        $secret = get_option('visma_test') ? WTV_Plugin::CLIENT_TEST_SECRET : WTV_Plugin::CLIENT_SECRET;
        $options['auth'] = [
            WTV_Plugin::CLIENT_ID,
            $secret
        ];
        $options['form_params'] = array(
            'code'          => $code,
            'grant_type'    => 'authorization_code',
            'redirect_uri'  => WTV_Auth::redirect_uri(),
        );

        $response = $client->post( 'token', $options );
        $contents = json_decode( $response->getBody() );
        write_log($contents);

        if( (int) $response->getStatusCode() >= 400 ){
            update_option( 'visma_needs_login', true );
            throw new Exception('Request failed' . $response->getStatusCode(), 1221 );
        }

        WTV_Auth::set_auth_data( $contents );
        return true;
    }

    /**
     * Returns Visma Authorize url
     * @return string
     */
    public static function authorize_url(){
        $uri = get_option('visma_test') ? self::BASE_TEST_URI : self::BASE_URI;
        return $uri . 'authorize?client_id=' . WTV_Plugin::CLIENT_ID . '&redirect_uri=' . self::redirect_uri() . '&scope=ea:accounting%20ea:purchase%20ea:sales%20ea:api%20offline_access&response_type=code';
    }

    /**
     * Returns Auth headers
     * @return array
     */
    public static function get_auth_headers( ){

        if( WTV_Auth::is_access_token_valid() ){
            return [
                'Authorization' => 'Bearer ' . get_option( 'visma_access_token' ),
                'Accept'        => 'application/json',
            ];
        }
        else{
            return [
                'Authorization' => 'Bearer ' . self::refresh_token(),
                'Accept'        => 'application/json',
            ];
        }

    }

    /**
     * Returns if Visma Access Token is valid
     * @throws Exception
     */
	public static function is_access_token_valid(){
        return intval( get_option( 'visma_expiry_time' ) ) > time();
	}

    /**
     * Sets Access Token, Refresh Token and expiry time in wp_options table
     *
     * @param array $data
     */
    public static function set_auth_data( $data ){

        if ( array_key_exists( 'access_token', $data ) ){
            update_option( 'visma_access_token', $data->access_token );
            delete_option( 'visma_needs_login' );
        }
        elseif ( array_key_exists( 'id_token', $data ) ){
            update_option( 'visma_access_token', $data->id_token );
            delete_option( 'visma_needs_login' );
        }

        update_option( 'visma_refresh_token', $data->refresh_token );
        update_option( 'visma_expiry_time', time() + ( 60 * 60 ) );
    }

    /**
     * Returns the Redirect URI used for authentication
     *
     * @return string
     */
    public static function redirect_uri(){
        return rest_url() . 'visma/callback';
	}

    /**
     * Refreshes AccessToken
     * @return string
     * @throws Exception
     */
    public static function refresh_token( ){
        $uri = get_option('visma_test') ? self::BASE_TEST_URI : self::BASE_URI;
        $client = new Client( [ 'base_uri' => $uri ] );
        $options = array();

        $secret = get_option('visma_test') ? WTV_Plugin::CLIENT_TEST_SECRET : WTV_Plugin::CLIENT_SECRET;
        $options['auth'] = [
            WTV_Plugin::CLIENT_ID,
            $secret
        ];
        $options['form_params'] = array(
            'refresh_token' => get_option( 'visma_refresh_token' ),
            'grant_type'    => 'refresh_token',
            'redirect_uri'  => WTV_Auth::redirect_uri(),
        );
        $response = $client->post( 'token', $options );
        $contents = json_decode( $response->getBody() );

        if( (int) $response->getStatusCode() >= 400 ){
            update_option( 'visma_needs_login', true );
            throw new Exception('Request failed' . $response->getStatusCode(), 1221 );
        }

        self::set_auth_data( $contents );
        return $contents->access_token;
    }
}