<?php

namespace includes\http;

require( WTV_PATH . '/vendor/autoload.php');

use GuzzleHttp\Client;
use Exception;
use includes\wetail\WTV_Credentials;
use Psr\Http\Message\ResponseInterface;
use GuzzleHttp\HandlerStack;
use Namshi\Cuzzle\Middleware\CurlFormatterMiddleware;
use Monolog\Logger;
use Monolog\Handler\TestHandler;

class WTV_Request
{

    const API_TEST_ENDPOINT = 'https://eaccountingapi-sandbox.test.vismaonline.com/';
    const API_ENDPOINT = 'https://eaccountingapi.vismaonline.com/';
    const ERROR_CODE_UNAUTHORIZED = 1;
    const ERROR_CODE_DUPLICATE = 2;
    const ERROR_CODE_BAD_REQUEST = 3;

    /**
     * Clean Data
     *
     * @param $data
     * @return mixed
     */
    public static function clean_data( $data )
    {
        if ( is_array($data)) {
            foreach ($data as $k => $v) {
                $data[$k] = self::clean_data($v);
            }
        } else if (is_string ($data)) {
            return htmlspecialchars( preg_replace('/[\/]/', '_', $data ) );
        }
        return $data;
    }

    /**
     * Extract error code from error response
     * @param $response
     * @return mixed
     */
	public static function get_error_code( $response )
	{
		if( ! empty( $response->ErrorInformation->Code ) )
			return $response->ErrorInformation->Code;

		if( ! empty( $response->ErrorInformation->code ) )
			return $response->ErrorInformation->code;

	}

    /**
     * Performs GET request
     * @param string $endpoint
     * @param array $params
     * @param string $version
     * @return mixed
     */
    public static function get( $endpoint, $params=[], $version='v2' ){

        $api_endpoint = get_option('visma_test') ? self::API_TEST_ENDPOINT : self::API_ENDPOINT;
        $client = new Client( ['base_uri' => $api_endpoint ]);

        $options = array(
            'headers' => WTV_Auth::get_auth_headers(),
            'http_errors' => false
        );
        $response = $client->get( $version . $endpoint, $options );

        return self::handle_response( $response, $endpoint, 'GET');
    }


    /**
     * Handle response
     *
     * @param ResponseInterface $response
     * @param $endpoint
     * @param $method
     * @param array $params
     * @return mixed
     * @throws Exception
     */
    public static function handle_response( $response, $endpoint, $method, $params=[]  )
    {
        if( (int) $response->getStatusCode() >= 400 ){
            if( (int) $response->getStatusCode() == 401 ){
                throw new Exception('AUTH FAILED, LOGIN AGAIN ' . $response->getStatusCode() . ' ' , self::ERROR_CODE_UNAUTHORIZED );
            }
            elseif( (int) $response->getStatusCode() == 409 ){
                throw new Exception('Request failed ' . $response->getStatusCode() . ' ' . $response->getBody()->getContents(), self::ERROR_CODE_DUPLICATE );
            }
            error_log('Request failed ' . $method . ' ' . $endpoint . ' ' . $response->getStatusCode() . ' ' . $response->getBody()->getContents() . ' ' . print_r( $response , true) . ' ' . print_r( $params , true));
            $error = json_decode( $response->getBody() );
            throw new Exception( $endpoint . ' ' . $error->DeveloperErrorMessage, $error->ErrorCode );
        }
        return json_decode( $response->getBody() );
    }

    /**
     * Performs POST request
     * @param string $endpoint
     * @param array $params
     * @param string $version
     * @return mixed
     * @throws Exception
     */
    public static function post( $endpoint, $params, $version='v2' ){

        if ( ! WTV_Credentials::check() ){
            throw new Exception( "Licensnyckel saknas eller är ogiltig." );
        }

        $api_endpoint = get_option('visma_test') ? self::API_TEST_ENDPOINT : self::API_ENDPOINT;
        $client = new Client( ['base_uri' => $api_endpoint ]);

        $options = array(
            'headers' => WTV_Auth::get_auth_headers(),
            'json' => $params,
            'http_errors' => false
        );

        write_log($endpoint);
        write_log($params);

        $response = $client->post( $version . $endpoint, $options );
        return self::handle_response( $response, $endpoint, 'POST',  $params );
    }

    /**
     * Perform PUT request
     * @param string $endpoint
     * @param array $params
     * @param string $version
     * @return mixed
     * @throws Exception
     */
    public static function put( $endpoint, $params, $version='v2' ){
        if ( ! WTV_Credentials::check() ){
            throw new Exception( "Licensnyckel saknas eller är ogiltig." );
        }


        $logger = new Logger('guzzle.to.curl'); //initialize the logger
        $testHandler = new TestHandler(); //test logger handler
        $logger->pushHandler($testHandler);

        $handler = HandlerStack::create();
        $handler->after('cookies', new CurlFormatterMiddleware($logger)); //add the cURL formatter middleware

        $api_endpoint = get_option('visma_test') ? self::API_TEST_ENDPOINT : self::API_ENDPOINT;
        $client  = new Client(['base_uri' => $api_endpoint, 'handler' => $handler]); //initialize a Guzzle client

        $options = array(
            'headers' => WTV_Auth::get_auth_headers(),
            'json' => $params,
            'http_errors' => false
        );

        $response = $client->put( $version . $endpoint, $options );

        return self::handle_response( $response, $endpoint, 'PUT', $params );
    }
}